@@ -25,30 +25,29 @@ use utf8;
 use Authen::Simple::PAM;
 use Authen::Simple::Password;
 use Carp;
-use CipUX 3.4.0.4;
+use CipUX 3.4.0.4;    ##no critic
 use CipUX::Task;
 use CipUX::RBAC::Simple;
 use Class::Std;
 use Data::Dumper;
-use Digest::MD5 qw(md5_hex);
 use English qw( -no_match_vars);
 use Frontier::Daemon;
 use Frontier::RPC2;
 use List::MoreUtils qw(any none);
+use Log::Dispatch;
 use Log::Log4perl qw(get_logger :levels);
 use Readonly;
+use Ticket::Simple;
 
 use base qw(CipUX::RPC);
-{             # BEGIN CLASS
+{    # BEGIN CLASS
 
     # CONSTRUCTOR
     # DESTRUCTOR
     # PRIVATE METHODS
     # * privacy cathegory 1
     #   - check_authentication
-    #   - create_ticket
     #   - is_ticket_bad
-    #   - calc_random_seed
     # * privacy cathegory 2
     #   - signal_handler
     #   - answer_requests
@@ -75,7 +74,7 @@ use base qw(CipUX::RPC);
     # - rpc_info
     # - rpc_intern
 
-    use version; our $VERSION = qv('3.4.0.6');
+    use version; our $VERSION = qv('3.4.0.8');
     use re 'taint';    # Keep data captured by parens tainted
     delete @ENV{qw(PATH IFS CDPATH ENV BASH_ENV)};    # Make %ENV safe
 
@@ -84,11 +83,7 @@ use base qw(CipUX::RPC);
     # +======================================================================+
     Readonly::Scalar my $EMPTY_STRING => q{};
     Readonly::Scalar my $SCRIPT       => 'CipUX::RPC::Server';
-    Readonly::Array my @PASSWD_CHARS =>
-        ( 'A' .. 'Z', 'a' .. 'z', 0 .. 9, qw(! @ $ % &) );
-    Readonly::Scalar my $TICKET_LENGTH => 128;
-    Readonly::Scalar my $SEED          => calc_random_seed();
-    Readonly::Scalar my $HEADER_HREF   => {
+    Readonly::Scalar my $HEADER_HREF  => {
         'cipux_version'  => '3.4.0.0',
         'server_name'    => 'cipux_rpcd',
         'server_version' => '3.4.0.0',
@@ -97,12 +92,11 @@ use base qw(CipUX::RPC);
         'server_cred'    => $EMPTY_STRING,
         'gmt_time'       => time,
     };
-    Readonly::Scalar my $DEF_TTL            => 1200;       # in sec
     Readonly::Scalar my $A_SHORT_WHILE      => 3;          # in sec
     Readonly::Scalar my $INTERN_ADMIN_GROUP => 'admin';    # last resort
 
     Readonly::Array my @ADMIN_GROUP_CLIENT =>
-        qw(cipux_rpc_test_client cipux_rpc_test_repetition cipuxpasswd);
+        qw(cipux_cat_web_module cipux_rpc_test_client cipux_rpc_test_repetition cipuxpasswd);
     Readonly::Array my @RPC_INTERN_CMD    => qw(rpc_intern);
     Readonly::Array my @RPC_INTERN_SUBCMD => qw(flush
         cat_module_cache_size
@@ -128,8 +122,8 @@ use base qw(CipUX::RPC);
     my $rpc_cfg_hr         = undef;  # undef means: load it at the first time.
     my $rpc                = undef;
     my $rbac               = undef;
+    my $ts                 = undef;
     my $time_to_die        = 0;      # shutdown server with SIG handler
-    my %time               = ();
 
     # +======================================================================+
     # || CONSTRUCTOR                                                        ||
@@ -151,6 +145,8 @@ use base qw(CipUX::RPC);
         $cipux_task_list_ar = $cipux_task->list_task();
         $rpc                = CipUX::RPC->new( { cache_dir => $cache_dir } );
         $rbac               = CipUX::RBAC::Simple->new();
+        my $ttl = $self->get_config('xml_rpc_ticket_ttl');
+        $ts = Ticket::Simple->new( { ttl => $ttl } );
 
         # +------------------------------------------------------------------+
         # | API
@@ -175,7 +171,7 @@ use base qw(CipUX::RPC);
         undef $rpc;
         undef $rbac;
         undef $time_to_die;
-        undef %time;
+        undef $ts;
 
         # +------------------------------------------------------------------+
         # | API
@@ -200,8 +196,6 @@ use base qw(CipUX::RPC);
         # | API
         my ( $self, $arg_r ) = @_;
 
-        #$self->var_dump( { var_r => \$arg_r, name => 'arg_r' } );
-
         my $login
             = exists $arg_r->{login}
             ? $self->l( $arg_r->{login} )
@@ -215,15 +209,18 @@ use base qw(CipUX::RPC);
         # +------------------------------------------------------------------+
         # | prepare
         my $logger = get_logger(__PACKAGE__);
-        $logger->debug('BEGIN');
 
+        # unsuccessfully authentication return 0 (FALSE)
         my $authenticated = 0;
 
         # check for empty password, return 0 (FALSE) if empty
         if ( $password eq $EMPTY_STRING ) {
             $logger->debug('Empty password');
-            $logger->debug('END');
-            return 0;
+            $logger->info("authentication not sucessfully for [$login]");
+
+            # unsuccessfully authentication return 0 (FALSE)
+            $authenticated = 0;
+            return $authenticated;    # we do not want further tests
         }
 
         $logger->debug('using PAM to authenticate login');
@@ -231,85 +228,16 @@ use base qw(CipUX::RPC);
         if ( $pam->authenticate( $login, $password ) ) {
 
             # successfully authentication return 1 (TRUE)
-            $logger->debug("authentication sucessfully for login [$login]");
+            $logger->info("authentication sucessfully for [$login]");
             $authenticated = 1;
 
         }
 
-        $logger->debug('END');
-
         # +------------------------------------------------------------------+
         # | API
         return $authenticated;
     }
 
-    # +~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~+
-    # :  create_ticket                                                    :
-    # +~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~+
-    # This sub can be used in 2 manners:
-    # (1):
-    # my $testticket = $self->create_ticket({login=>$l,test=>1});
-    # (0)
-    # my $ticket = $self->create_ticket({login=>$l,test=>0});
-    # my $ticket = $self->create_ticket({login=>$l});
-    # This form also stores the valid untill time global for $login
-    sub create_ticket : PRIVATE {
-
-        # +------------------------------------------------------------------+
-        # | API
-        my ( $self, $arg_r ) = @_;
-
-        my $logger = get_logger(__PACKAGE__);
-        $logger->debug('BEGIN');
-
-        my $login
-            = exists $arg_r->{login}
-            ? $self->l( $arg_r->{login} )
-            : $self->perr('login create_ticket');
-
-        my $test
-            = (     exists $arg_r->{test}
-                and defined $arg_r->{test}
-                and $arg_r->{test} ) ? 1 : 0;
-
-        # +------------------------------------------------------------------+
-        # | prepare
-        $logger->debug("input parameter login: [$login]");
-        $logger->debug("input parameter test: [$test]");
-        $logger->debug("global parameter SEED: [$SEED]");
-
-        my $ttl
-            = ( $self->get_config('xml_rpc_ticket_ttl') )
-            ? $self->get_config('xml_rpc_ticket_ttl')
-            : $DEF_TTL;
-        $logger->debug( 'ttl: ', $ttl );
-
-        # main
-        if ($test) {
-            $logger->debug('This is just a test!');
-        }
-        else {
-            $logger->debug('This is not a test!');
-            $logger->debug( 'old time: ', $time{$login} );
-            my $time = time;
-            $logger->debug( 'actual time: ', $time{$login} );
-            $time{$login} = $time + $ttl;
-            $logger->debug( 'new time: ', $time{$login} );
-        }
-
-        my $ticket
-            = md5_hex( join $EMPTY_STRING, $time{$login}, $SEED, $login );
-
-        $logger->debug( 'created ticket: ', $ticket );
-
-        $logger->debug('END');
-
-        # +------------------------------------------------------------------+
-        # | API
-        return $ticket;
-
-    }
-
     # +~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~+
     # :  is_ticket_bad                                                       :
     # +~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~+
@@ -319,7 +247,6 @@ use base qw(CipUX::RPC);
         # | API
         my ( $self, $arg_r ) = @_;
 
-        #$self->var_dump( { var_r => \$arg_r, name => 'arg_r' } );
         my $login
             = exists $arg_r->{login}
             ? $self->l( $arg_r->{login} )
@@ -338,103 +265,35 @@ use base qw(CipUX::RPC);
         # +------------------------------------------------------------------+
         # | prepare
         my $logger = get_logger(__PACKAGE__);
-        $logger->debug('BEGIN');
-        $logger->debug( 'login: ',     $login );
-        $logger->debug( 'ticket: ',    'not printed' );
-        $logger->debug( 'answer_hr: ', 'not printed' );
+        $logger->debug( 'got login: ',     $login );
+        $logger->debug( 'got ticket: ',    $ticket );
+        $logger->debug( 'got answer_hr: ', 'not printed' );
 
         # +------------------------------------------------------------------+
         # | main
-        my $negmsg = 'The ticket check failed! The ticked is not valid.';
+        if (
+            not $ts->is_ticket_valid_now(
+                { login => $login, ticket => $ticket }
+            )
+            )
+        {
+            $logger->warn("ticket [$ticket] is not valid for [$login] now");
 
-        # 2 NO TIME (check if we have a time stamp, if not: noughty noughty)
-        if ( not defined $time{$login} ) {
+            my $msg = 'Your ticket is not valid!';
+            $msg .= ' This can have serveral reasons:';
+            $msg .= ' (1) the ticket expired. (2) an error in programming  ';
+            $msg .= ' (3) you never logged in';
 
-            $logger->debug( 'NO TIME login: ', $login );
-            $logger->debug( 'time hash: ',
-                { filter => \&Dumper, value => \%time } );
-            $logger->debug('NO TIME problem 2 (reset cookies)');
-
-            my $msg = 'No stored time can be found for the ticket.';
-            $msg .= ' Probably you logged out before.';
-            $msg .= ' Please log in to continue!';
-            $answer_hr->{msg} = $negmsg . $msg;
-            $answer_hr->{ticket}
-                = 'test';    # reset cookie => print login dialog
-            $answer_hr->{login} = 'test'; # reset cookie => print login dialog
-            $answer_hr->{problem} = 2;        # NO TIME
-            $answer_hr->{ltarget} = 'NULL';
-
-            $logger->debug('END');
-
-            # +--------------------------------------------------------------+
-            # | API
-            return { result => 1, answer_hr => $answer_hr };
-
-        }
-
-        my $now = time;
-        $logger->debug( 'the time is now: ',    $now );
-        $logger->debug( 'ticket valid until: ', $time{$login} );
-
-        # 3 TIME OUT (time out, game over!)
-        if ( $time{$login} < $now ) {
-
-            $logger->debug( 'TIME OUT login: ',       $login );
-            $logger->debug( 'TIME OUT valid until: ', $time{$login} );
-            $logger->debug( 'TIME OUT time now: :',   $now );
-            $logger->debug('TIME OUT Ticket is bad? YES!!');
-            $logger->debug('TIME OUT problem 3: time out');
-
-            my $msg = ' It was a timeout, because the ticked time is too';
-            $msg .= ' old. Please log in again!';
-            $answer_hr->{msg}     = $negmsg . $msg;
+            # reset cookie => print login dialog
             $answer_hr->{ticket}  = 'test';
             $answer_hr->{login}   = 'test';
-            $answer_hr->{problem} = 3;                # time out
+            $answer_hr->{msg}     = $msg;
+            $answer_hr->{problem} = 2;        # NO TIME
             $answer_hr->{ltarget} = 'NULL';
-
-            $logger->debug('END');
-
-            # +--------------------------------------------------------------+
-            # | API
             return { result => 1, answer_hr => $answer_hr };
         }
 
-        # recreate ticket to be sure if given ticket is valid
-        my $recreated
-            = $self->create_ticket( { login => $login, test => 1 } );
-
-        # 4 TIME LOGIN MISMATCH
-        if ( $ticket ne $recreated ) {
-
-            my $msg = 'The ticket is not valid. The time and login';
-            $msg .= ' do not match. Do you try to use two connections?';
-            $msg .= ' To continue, log in only once!';
-
-            $logger->debug( 'TIME LOGIN MISM. login:       ', $login );
-            $logger->debug( 'TIME LOGIN MISM. valid until: ', $time{$login} );
-            $logger->debug( 'TIME LOGIN MISM. time now:    ', $now );
-            $logger->debug( 'TIME LOGIN MISM. old ticket:  ', $ticket );
-            $logger->debug( 'TIME LOGIN MISM. test ticket: ', $recreated );
-            $logger->debug('Is the ticked bad? YES!! ');
-            $logger->debug('problem 4: time login mismatch');
-
-            $answer_hr->{msg}     = $negmsg . $msg;
-            $answer_hr->{ticket}  = 'test';           # time login mismatch
-            $answer_hr->{login}   = 'test';           # time login mismatch
-            $answer_hr->{problem} = 4;                # time login mismatch
-            $answer_hr->{ltarget} = 'NULL';
-
-            $logger->debug('END');
-
-            # +--------------------------------------------------------------+
-            # | API
-            return { result => 1, answer_hr => $answer_hr };
-
-        }
-
-        # OK, time and ticket ans SEED match, must be ours!
+        # OK, time and ticket an SEED match, must be ours!
 
         $answer_hr->{msg}     = $EMPTY_STRING;
         $answer_hr->{problem} = 0;               # no problem
@@ -442,10 +301,7 @@ use base qw(CipUX::RPC);
         $logger->debug('Is the ticket bad? NO!! Ticket is not bad.');
         $logger->debug('problem 0: no problem');
 
-        #$ticket = $self->create_ticket( { login => $login } );
-        #$answer_hr->{ticket}  = $ticket;
-
-        $logger->debug('END');
+        $logger->warn("ticket [$ticket] is good for [$login]");
 
         # +------------------------------------------------------------------+
         # | API
@@ -453,36 +309,6 @@ use base qw(CipUX::RPC);
 
     } ## end sub is_ticket_bad
 
-    # +~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~+
-    # :  calc_random_seed                                                    :
-    # +~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~+
-    sub calc_random_seed : PRIVATE {
-
-        # +------------------------------------------------------------------+
-        # | main
-        my $logger = get_logger(__PACKAGE__);
-        $logger->debug('BEGIN');
-
-        # Calculating secret random seed for this session
-        # "S ISp&FtR0z$EU!We8DvpUzC26D0RE1pVW8vSXp9at5RUwXk
-        # WesmQvJY!w!LrLHdo^wB7f6lr7U9PGPTYhxTI!PhKjXhMmZZK
-        # ckIi^Qbl&g^$Qir!9S5LIoo!J1bX*OHVw"
-
-        srand;
-
-        my @chars = @PASSWD_CHARS;
-
-        my $seed = join q{},
-            @chars[ map { rand @chars } ( 1 .. $TICKET_LENGTH ) ];
-
-        $logger->debug('END');
-
-        # +------------------------------------------------------------------+
-        # | API
-        return $seed;
-
-    } ## end sub calc_random_seed
-
     # +----------------------------------------------------------------------+
     # |  PRIVACY CATEGORY 2 (might change to public if needed)               |
     # +----------------------------------------------------------------------+
@@ -495,12 +321,9 @@ use base qw(CipUX::RPC);
         # +------------------------------------------------------------------+
         # | prepare
         my $logger = get_logger(__PACKAGE__);
-        $logger->debug('BEGIN');
 
         $time_to_die = 1;
 
-        $logger->debug('END');
-
         # +------------------------------------------------------------------+
         # | API
         return;
@@ -515,7 +338,6 @@ use base qw(CipUX::RPC);
         my ( $self, $arg_r ) = @_;
 
         my $logger = get_logger(__PACKAGE__);
-        $logger->debug('BEGIN');
 
         my $port
             = exists $arg_r->{port}
@@ -568,7 +390,7 @@ use base qw(CipUX::RPC);
             $msg .= 'the port is in use. If you see this message only ';
             $msg .= 'one time everything is OK.';
 
-            $logger->debug( $msg, $z );
+            $logger->info( $msg, $z );
             $z++;
 
             # LocalPort => "https(8001)",
@@ -591,8 +413,6 @@ use base qw(CipUX::RPC);
             $logger->debug( 'daemon: ', $daemon );
         } ## end while ( not $time_to_die )
 
-        $logger->debug('END');
-
         # +------------------------------------------------------------------+
         # | API
         return 1;
@@ -621,28 +441,26 @@ use base qw(CipUX::RPC);
         # +------------------------------------------------------------------+
         # | main
         my $logger = get_logger(__PACKAGE__);
-        $logger->debug('BEGIN');
-        $logger->debug("login [$login]");
-        $logger->debug("cmd [$cmd]");
 
         if ( $login eq $EMPTY_STRING ) {
-            $logger->debug("NO ACCESS login is empty");
+            $logger->info("Access DENIED for [$login] to [$cmd]");
+            $logger->debug('Empty login: Access DENIED');
             return 0;
         }
 
         if ( $cmd eq 'cipux_task_sum' ) {
+            $logger->info("Access GRANTED for [$login] to [$cmd]!");
             return 1;
         }
         elsif ( $rbac->access_to_task( { task => $cmd, user => $login } ) ) {
-            $logger->debug('ACCESS');
+            $logger->info("Access GRANTED for [$login] to [$cmd]");
             return 1;
         }
         else {
-            $logger->debug('NO ACCESS');
+            $logger->info("Access DENIED for [$login] to [$cmd]");
+            $logger->debug('Empty login: Access DENIED');
         }
 
-        $logger->debug('END');
-
         # +------------------------------------------------------------------+
         # | API
         return 0;
@@ -670,7 +488,6 @@ use base qw(CipUX::RPC);
         # +------------------------------------------------------------------+
         # | main
         my $logger = get_logger(__PACKAGE__);
-        $logger->debug('BEGIN');
         $logger->debug("login [$login]");
         $logger->debug("cmd [$cmd]");
 
@@ -690,8 +507,6 @@ use base qw(CipUX::RPC);
             $logger->debug('NO ACCESS');
         }
 
-        $logger->debug('END');
-
         # +------------------------------------------------------------------+
         # | API
         return 0;
@@ -719,7 +534,6 @@ use base qw(CipUX::RPC);
         # +------------------------------------------------------------------+
         # | main
         my $logger = get_logger(__PACKAGE__);
-        $logger->debug('BEGIN');
         $logger->debug("login [$login]");
         $logger->debug("cmd [$cmd]");
 
@@ -744,8 +558,6 @@ use base qw(CipUX::RPC);
             $logger->debug('NO ACCESS');
         }
 
-        $logger->debug('END');
-
         # +------------------------------------------------------------------+
         # | API
         return 0;
@@ -761,7 +573,6 @@ use base qw(CipUX::RPC);
         my ( $self, $arg_r ) = @_;
 
         my $logger = get_logger(__PACKAGE__);
-        $logger->debug('BEGIN');
 
         my $login
             = exists $arg_r->{login}
@@ -833,7 +644,6 @@ use base qw(CipUX::RPC);
             'rpc_intern' => \&check_access_to_rpc_intern,
         };
         my $logger = get_logger(__PACKAGE__);
-        $logger->debug('BEGIN');
         $logger->debug("login [$login]");
         $logger->debug("realm [$realm]");
         if ( not exists $check_access_disp_hr->{$realm} ) {
@@ -860,8 +670,10 @@ use base qw(CipUX::RPC);
 
         # we do not trust the "from" param.
         my $from
-            = ( $subcmd eq 'task_access' or $subcmd eq 'task_access_survey' )
-            ? $login
+            = (
+                   $subcmd eq 'task_access'
+                or $subcmd eq 'task_access_survey'
+            ) ? $login
             : (    $subcmd eq 'user_task_access'
                 or $subcmd eq 'user_task_access_survey'
                 or $subcmd eq 'user_cat_module_access'
@@ -904,7 +716,6 @@ use base qw(CipUX::RPC);
         # +------------------------------------------------------------------+
         # | prepare
         my $logger = get_logger(__PACKAGE__);
-        $logger->debug('BEGIN');
 
         my %task     = ();
         my $return_r = $cipux_task->task(
@@ -928,7 +739,9 @@ use base qw(CipUX::RPC);
         #     'status' => 'OK',
         #     'type' => 'href'
         # };
-        if ( exists $return_r->{status} and $return_r->{status} eq 'OK' ) {
+        if ( exists $return_r->{status}
+            and $return_r->{status} eq 'OK' )
+        {
             my @task = keys %{ $return_r->{taskres_r} };
             %task = map { $_ => 1 } @task;
         }
@@ -980,7 +793,6 @@ use base qw(CipUX::RPC);
         # +------------------------------------------------------------------+
         # | prepare
         my $logger = get_logger(__PACKAGE__);
-        $logger->debug('BEGIN');
 
         my %cat      = ();
         my $return_r = $cipux_task->task(
@@ -1004,7 +816,9 @@ use base qw(CipUX::RPC);
         #     'status' => 'OK',
         #     'type' => 'href'
         # };
-        if ( exists $return_r->{status} and $return_r->{status} eq 'OK' ) {
+        if ( exists $return_r->{status}
+            and $return_r->{status} eq 'OK' )
+        {
             my @cat = keys %{ $return_r->{taskres_r} };
             %cat = map { $_ => 1 } @cat;
         }
@@ -1014,10 +828,10 @@ use base qw(CipUX::RPC);
 
         foreach my $mod (@ADMIN_GROUP_CLIENT) {
             if ( $cat{$mod} ) {
-                $logger->debug("task exists [$mod]");
+                $logger->debug("module exists [$mod]");
             }
             else {
-                $logger->debug("task [$mod] do not exist");
+                $logger->debug("module [$mod] do not exist");
 
                 # default 'admin' (could be other)
                 my $role
@@ -1071,6 +885,20 @@ use base qw(CipUX::RPC);
     # || PUBLIC METHODS                                                     ||
     # +======================================================================+
 
+    # +----------------------------------------------------------------------+
+    # |  log2syslog                                                          |
+    # +----------------------------------------------------------------------+
+    sub log2syslog {
+
+        my $message = shift;
+
+        my $log = Log::Dispatch->new(
+            outputs => [ [ 'Syslog', min_level => 'info', ] ] );
+        $log->emerg($message);
+
+        return;
+    }
+
     # +----------------------------------------------------------------------+
     # |  get_config                                                          |
     # +----------------------------------------------------------------------+
@@ -1081,7 +909,6 @@ use base qw(CipUX::RPC);
         my ( $self, $key ) = @_;
 
         my $logger = get_logger(__PACKAGE__);
-        $logger->debug('BEGIN');
         $logger->debug("asked for configuration key [$key]");
 
         # +------------------------------------------------------------------+
@@ -1094,8 +921,6 @@ use base qw(CipUX::RPC);
 
         my $return = $rpc_cfg_hr->{$key};
 
-        $logger->debug('END');
-
         # +------------------------------------------------------------------+
         # | API
         return $return;
@@ -1112,7 +937,6 @@ use base qw(CipUX::RPC);
         my ( $self, $arg_r ) = @_;
 
         my $logger = get_logger(__PACKAGE__);
-        $logger->debug('BEGIN');
 
         my $scope
             = exists $arg_r->{scope}
@@ -1129,8 +953,6 @@ use base qw(CipUX::RPC);
         push @cmd, qw( ping version sum login logout session ttl);
         push @cmd, qw( task rpc_info rpc_intern );
 
-        $logger->debug('END');
-
         # +------------------------------------------------------------------+
         # | API
         return @cmd;
@@ -1147,7 +969,6 @@ use base qw(CipUX::RPC);
         my ( $self, $arg_r ) = @_;
 
         my $logger = get_logger(__PACKAGE__);
-        $logger->debug('BEGIN');
 
         # +------------------------------------------------------------------+
         # | main
@@ -1212,7 +1033,7 @@ use base qw(CipUX::RPC);
         #  $logger->debug($msg.q{: }, $xml_rpc_ticket_ttl  ,"\n");
 
         # as everybody, who got killed, we have to sleep 3 seconds
-        # to regenerate our self to be awake again ;-)
+        # to regenerate our self to awake again ;-)
         sleep $A_SHORT_WHILE;
 
         # check all given tasks.
@@ -1239,14 +1060,12 @@ use base qw(CipUX::RPC);
             task    => sub {
 
                 #my $logger = get_logger(__PACKAGE__);
-                #$logger->debug('BEGIN');
                 #foreach my $x (@_) {
                 #    $logger->debug("X [$x]");
                 #    foreach my $h ( keys %{$x} ) {
                 #        $logger->debug("h [$h] -> [$x->{$h}]");
                 #    }
                 #}
-                #$logger->debug('END');
                 if ( $self->check_access_to_task(@_) ) {
                     $self->rpc_task(@_);
                 }
@@ -1272,8 +1091,6 @@ use base qw(CipUX::RPC);
             }
         );
 
-        $logger->debug('END');
-
         # +------------------------------------------------------------------+
         # | API
         return 1;
@@ -1294,7 +1111,6 @@ use base qw(CipUX::RPC);
         my ( $self, $arg_r ) = @_;
 
         my $logger = get_logger(__PACKAGE__);
-        $logger->debug('BEGIN');
 
         my $header_hr
             = exists $arg_r->{header_hr}
@@ -1337,8 +1153,6 @@ use base qw(CipUX::RPC);
             ltarget   => 'NULL',
         };
 
-        $logger->debug('END');
-
         # +------------------------------------------------------------------+
         # | API
         return $answer_hr;
@@ -1355,7 +1169,6 @@ use base qw(CipUX::RPC);
         my ( $self, $arg_r ) = @_;
 
         my $logger = get_logger(__PACKAGE__);
-        $logger->debug('BEGIN');
 
         my $header_hr
             = exists $arg_r->{header_hr}
@@ -1403,8 +1216,6 @@ use base qw(CipUX::RPC);
             ltarget   => 'NULL',
         };
 
-        $logger->debug('END');
-
         # +------------------------------------------------------------------+
         # | API
         return $answer_hr;
@@ -1421,7 +1232,6 @@ use base qw(CipUX::RPC);
         my ( $self, $arg_r ) = @_;
 
         my $logger = get_logger(__PACKAGE__);
-        $logger->debug('BEGIN');
 
         my $header_hr
             = exists $arg_r->{header_hr}
@@ -1479,8 +1289,6 @@ use base qw(CipUX::RPC);
             ltarget   => 'NULL',
         };
 
-        $logger->debug('END');
-
         # +------------------------------------------------------------------+
         # | API
         return $answer_hr;
@@ -1497,7 +1305,6 @@ use base qw(CipUX::RPC);
         my ( $self, $arg_r ) = @_;
 
         my $logger = get_logger(__PACKAGE__);
-        $logger->debug('BEGIN');
 
         my $header_hr
             = exists $arg_r->{header_hr}
@@ -1560,8 +1367,6 @@ use base qw(CipUX::RPC);
             my $msg = 'empty password given.';
             $logger->debug($msg);
 
-            $logger->debug('END');
-
            # +---------------------------------------------------------------+
            # | API
             return $answer_hr;
@@ -1579,10 +1384,14 @@ use base qw(CipUX::RPC);
             $logger->debug("login is_authenticated:  $login");
 
             #$logger->debug("old login ticket: [$ticket]");
-            $ticket = $self->create_ticket( { login => $login } );
+            # create new ticket, no test
+            my ( $ticket, $valid )
+                = $ts->create_ticket( { login => $login } );
+            $ts->store_ticket(
+                { login => $login, ticket => $ticket, valid => $valid } );
 
             #$logger->debug("new login ticket: [$ticket]");
-            $logger->debug('got ticket. not printed here.');
+            $logger->debug( 'ticket: ', $ticket );
 
             # differnt answer
             $answer_hr->{status}   = 'TRUE';
@@ -1594,8 +1403,6 @@ use base qw(CipUX::RPC);
             };
             $answer_hr->{ltarget} = 'NULL';
 
-            $logger->debug('END');
-
           # +----------------------------------------------------------------+
           # | API
             return $answer_hr;
@@ -1604,15 +1411,11 @@ use base qw(CipUX::RPC);
             my $msg = 'WARN password is valid NOT for login login ';
             $logger->debug( $msg . q{: }, $login );
 
-            $logger->debug('END');
-
           # +----------------------------------------------------------------+
           # | API
             return $answer_hr;
         }
 
-        $logger->debug('END');
-
         # +------------------------------------------------------------------+
         # | API
         return $answer_hr;
@@ -1629,7 +1432,6 @@ use base qw(CipUX::RPC);
         my ( $self, $arg_r ) = @_;
 
         my $logger = get_logger(__PACKAGE__);
-        $logger->debug('BEGIN');
 
         my $header_hr
             = exists $arg_r->{header_hr}
@@ -1676,7 +1478,6 @@ use base qw(CipUX::RPC);
             {
                 login     => $login,
                 ticket    => $ticket,
-                time      => $time{$login},
                 answer_hr => $answer_hr
             }
         );
@@ -1685,20 +1486,17 @@ use base qw(CipUX::RPC);
         # | API
         if ( $bad_hr->{result} ) {
             $logger->debug('bad result');
-            $logger->debug('END');
             return $bad_hr->{answer_hr};
         }
 
         # main
-        undef $time{$login};
+        $ts->wipe_ticket( { login => $login } );
 
         # answer
         $answer_hr->{status}   = 'TRUE';
         $answer_hr->{type}     = 'href';
         $answer_hr->{cmdres_r} = {};
 
-        $logger->debug('END');
-
         # +------------------------------------------------------------------+
         # | API
         return $answer_hr;
@@ -1715,7 +1513,6 @@ use base qw(CipUX::RPC);
         my ( $self, $arg_r ) = @_;
 
         my $logger = get_logger(__PACKAGE__);
-        $logger->debug('BEGIN');
 
         my $header_hr
             = exists $arg_r->{header_hr}
@@ -1762,30 +1559,26 @@ use base qw(CipUX::RPC);
             {
                 login     => $login,
                 ticket    => $ticket,
-                'time'    => $time{$login},
                 answer_hr => $answer_hr
             }
         );
 
         if ( $bad_hr->{result} ) {
             $logger->debug('bad result');
-            $logger->debug('END');
 
             # +--------------------------------------------------------------+
             # | API
             return $bad_hr->{answer_hr};
         }
 
-        # main (renew_ticket)
-        $ticket = $self->create_ticket( { login => $login } );
+        my ( $t, $v ) = $ts->create_ticket( { login => $login } );
+        $ts->store_ticket( { login => $login, ticket => $t, valid => $v } );
 
         # answer
-        $answer_hr->{ticket}   = $ticket;
+        #$answer_hr->{ticket}   = $t;
         $answer_hr->{status}   = 'TRUE';
         $answer_hr->{type}     = 'href';
-        $answer_hr->{cmdres_r} = {};
-
-        $logger->debug('END');
+        $answer_hr->{cmdres_r} = { ticket => $t, };
 
         # +------------------------------------------------------------------+
         # | API
@@ -1803,7 +1596,6 @@ use base qw(CipUX::RPC);
         my ( $self, $arg_r ) = @_;
 
         my $logger = get_logger(__PACKAGE__);
-        $logger->debug('BEGIN');
 
         my $header_hr
             = exists $arg_r->{header_hr}
@@ -1846,32 +1638,24 @@ use base qw(CipUX::RPC);
         };
 
         # check the ticket
-        $logger->debug( 'stored time is: ', $time{$login} );
         my $bad_hr = $self->is_ticket_bad(
             {
                 login     => $login,
                 ticket    => $ticket,
-                time      => $time{$login},
                 answer_hr => $answer_hr
             }
         );
-        $logger->debug( 'stored time is: ', $time{$login} );
 
         if ( $bad_hr->{result} ) {
             $logger->debug('bad result');
-            $logger->debug('END');
 
             # +--------------------------------------------------------------+
             # | API
             return $bad_hr->{answer_hr};
         }
-        $logger->debug( 'stored time is: ', $time{$login} );
-
-        # renew_ticket
-        #$ticket = $self->create_ticket( { login => $login } );
 
         # main
-        my $ttl = $self->get_config('xml_rpc_ticket_ttl');
+        my $ttl = $ts->get_ttl;
 
         # answer
         $answer_hr->{login}    = $login;
@@ -1880,8 +1664,6 @@ use base qw(CipUX::RPC);
         $answer_hr->{type}     = 'href';
         $answer_hr->{cmdres_r} = { ttl => $ttl };
 
-        $logger->debug('END');
-
         # +------------------------------------------------------------------+
         # | API
         return $answer_hr;
@@ -1898,7 +1680,6 @@ use base qw(CipUX::RPC);
         my ( $self, $arg_r ) = @_;
 
         my $logger = get_logger(__PACKAGE__);
-        $logger->debug('BEGIN');
 
         my $header_hr
             = exists $arg_r->{header_hr}
@@ -1926,7 +1707,8 @@ use base qw(CipUX::RPC);
             : $self->perr('param_hr');
 
         # API layer 2
-        my $object = $self->l( $param_hr->{object} ) || $EMPTY_STRING;
+        my $object = $self->l( $param_hr->{object} )
+            || $EMPTY_STRING;
 
         # +------------------------------------------------------------------+
         # | main
@@ -1957,7 +1739,6 @@ use base qw(CipUX::RPC);
             $answer_hr->{msg} = $cmd;
 
             $logger->debug($cmd);
-            $logger->debug('END');
 
             # +--------------------------------------------------------------+
             # | API
@@ -1972,14 +1753,12 @@ use base qw(CipUX::RPC);
                 {
                     login     => $login,
                     ticket    => $ticket,
-                    'time'    => $time{$login},
                     answer_hr => $answer_hr
                 }
             );
 
             if ( $bad_hr->{result} ) {
                 $logger->debug('bad result');
-                $logger->debug('END');
 
                 # +----------------------------------------------------------+
                 # | API
@@ -1988,9 +1767,6 @@ use base qw(CipUX::RPC);
 
             $logger->debug("create new ticket for $login");
 
-            # renew_ticket
-            $ticket = $self->create_ticket( { login => $login } );
-
         }
 
         if ( $cmd eq 'cipux_task_change_own_password_interactive' ) {
@@ -2046,7 +1822,6 @@ use base qw(CipUX::RPC);
         else {
             $logger->debug('got something else (not a hash) as answer_hr');
         }
-        $logger->debug('END');
 
         # +------------------------------------------------------------------+
         # | API
@@ -2109,9 +1884,8 @@ use base qw(CipUX::RPC);
         # | main
         my $logger = get_logger(__PACKAGE__);
 
-        $logger->debug('BEGIN');
         $logger->debug("> login:     $login");
-        $logger->debug('> ticket:    (not printed');
+        $logger->debug( 'ticket: ', $ticket );
         $logger->debug('> header_hr: (not printed)');
         $logger->debug( '> param_hr',
             { filter => \&Dumper, value => $param_hr } );
@@ -2157,7 +1931,9 @@ use base qw(CipUX::RPC);
         my $status = 'FALSE';
 
         # -------------------- check_access_to_task ------------------
-        if ( $subcmd eq 'task_access' or $subcmd eq 'task_access_survey' ) {
+        if (   $subcmd eq 'task_access'
+            or $subcmd eq 'task_access_survey' )
+        {
             $cmdres_r = $self->evaluate_access(
                 {
                     realm    => 'task',
@@ -2248,9 +2024,8 @@ use base qw(CipUX::RPC);
         # | main
         my $logger = get_logger(__PACKAGE__);
 
-        $logger->debug('BEGIN');
         $logger->debug("> login:     $login");
-        $logger->debug('> ticket:    (not printed');
+        $logger->debug( '> ticket: ', $ticket );
         $logger->debug('> header_hr: (not printed)');
         $logger->debug( '> param_hr',
             { filter => \&Dumper, value => $param_hr } );
@@ -2359,8 +2134,6 @@ use base qw(CipUX::RPC);
             ltarget   => 'NULL',
         };
 
-        $logger->debug('END');
-
         # +------------------------------------------------------------------+
         # | API
         return $answer_hr;
@@ -2374,13 +2147,15 @@ __END__
 
 =pod
 
+=for stopwords RPC CipUX rpc Destructor login subcmd syslog logout ttl TTL RBAC uid TODO cipux ini webpage Kuelker log2syslog cipux-rpc.ini CipUX::RPC::Server XML-RPC
+
 =head1 NAME
 
 CipUX::RPC::Server - RPC server class for CipUX
 
 =head1 VERSION
 
-version 3.4.0.6
+version 3.4.0.8
 
 =head1 SYNOPSIS
 
@@ -2420,18 +2195,10 @@ B<Syntax:>
 
 Check weather the login has access or not.
 
-=head2 create_ticket
-
-Create a ticket.
-
 =head2 is_ticket_bad
 
 Return 1 if a ticket is OK otherwise 0;
 
-=head2 calc_random_seed
-
-Calculate a long random number. This is used in the ticket.
-
 =head2 signal_handler
 
 Install signal_handler alias time to die.
@@ -2459,6 +2226,7 @@ B<Syntax:>
 
 Check the access for login to a task.
 
+
 =head2 check_access_to_rpc_intern
 
 Check the access for login to the rpc_intern section.
@@ -2485,6 +2253,10 @@ Check and update tasks entries.
 
 Check and update CAT modules entries.
 
+=head2 log2syslog
+
+Log a given message to syslog.
+
 =head2 get_config
 
 Return the value for a given configuration variable.
@@ -2531,7 +2303,7 @@ Check the ticket and if it is valid update and return a new ticket.
 
 =head2 ttl
 
-Return the Time To Live. Default 1200 seconds.
+Return the Time To Live. Default 900 seconds.
 
 =head2 rpc_task
 
@@ -2664,14 +2436,15 @@ cipux-rpc.ini
  CipUX::RBAC::Simple
  Class::Std
  Data::Dumper
- Digest::MD5
  English
  Frontier::Daemon
  Frontier::RPC2
  List::MoreUtils
  Log::Log4perl
+ Log::Dispatch
  POSIX
  Readonly
+ Ticket::Simple
 
 =head1 INCOMPATIBILITIES
 
