@@ -1,13 +1,10 @@
 # +==========================================================================+
 # || CipUX::RPC::Test::Client                                               ||
 # ||                                                                        ||
-# || CipUX RPC Test Client                                                  ||
-# ||                                                                        ||
-# || Copyright (C) 2008 - 2009 by Christian Kuelker. All rights reserved!   ||
+# || Copyright (C) 2008 - 2010 by Christian Kuelker. All rights reserved!   ||
 # ||                                                                        ||
 # || License: GNU GPL - GNU General Public License - version 2              ||
 # ||          or (at your opinion) any later version.                       ||
-# ||                                                                        ||
 # +==========================================================================+
 # ID:       $Id$
 # Revision: $Revision$
@@ -33,15 +30,13 @@ use Log::Log4perl qw(get_logger :levels);
 use Pod::Usage;
 use Readonly;
 
-{    # BEGIN INSIDE-OUT CLASS
+{
 
-    use version; our $VERSION = qv('3.4.0.6');
+    use version; our $VERSION = qv('3.4.0.8');
     use re 'taint';    # Keep data captured by parens tainted
     delete @ENV{qw(PATH IFS CDPATH ENV BASH_ENV)};    # Make %ENV safe
 
-    # +======================================================================+
-    # || CONSTANTS                                                          ||
-    # +======================================================================+
+    # CONSTANTS
     Readonly::Scalar my $SCRIPT       => 'CipUX::RPC::Test::Client';
     Readonly::Scalar my $EMPTY_STRING => q{};
     Readonly::Scalar my $LINEWIDTH    => 78;
@@ -91,18 +86,11 @@ use Readonly;
         ? '/etc/cipux/log4perl.conf'
         : $EMPTY_STRING;
 
-    # +======================================================================+
-    # || OBJECT                                                             ||
-    # +======================================================================+
-
-    # register client with name:
-    #     name : cipux_rpc_test_client
+    # OBJECT
+    # register client with name: cipux_rpc_test_client
     my %name_of : ATTR( init_arg => 'name');
 
-    # +=====================================================================+
-    # || GLOBAL                                                            ||
-    # +=====================================================================+
-
+    # GLOBAL
     my $L = q{=} x $LINEWIDTH;
     $L .= "\n";
     my $url             = $EMPTY_STRING;
@@ -148,9 +136,7 @@ use Readonly;
             $self->exc( { msg => 'unknown action', value => $run_action } );
         }
 
-      # +====================================================================+
-      # || ENVIRONMENT                                                      ||
-      # +====================================================================+
+        # ENVIRONMENT
         Getopt::Long::Configure('bundling');
 
         # first thing will be defined: debug,help,version
@@ -176,7 +162,6 @@ use Readonly;
         my $logger = get_logger(__PACKAGE__);
 
         my $date = UnixDate( 'today', '%O' );
-        $logger->debug('BEGIN');
         $logger->debug("    CipUX : $VERSION   ");
         $logger->debug("    date  : $date");
         $logger->debug("    action: $run_action");
@@ -223,7 +208,7 @@ use Readonly;
             = ( exists $config_hr->{xml_rpc_ttl}
                 and defined $config_hr->{xml_rpc_ttl} )
             ? $config_hr->{xml_rpc_ttl}
-            : 1200;
+            : 600;
 
         $logger->debug("    host  : $host");
         $logger->debug("    port  : $port");
@@ -250,10 +235,7 @@ use Readonly;
             }
         );
 
-      # +====================================================================+
-      # || MAIN                                                             ||
-      # +====================================================================+
-
+        # MAIN
         $self->test_ping;
         $self->test_version();
         $self->test_sum1();
@@ -276,21 +258,14 @@ use Readonly;
         $self->test_plain_entity();
         $self->test_summary();
 
-        $logger->debug('END');
-
         return;
     }
 
-   # +=======================================================================+
-   # || ping                                                                ||
-   # +=======================================================================+
-
     sub test_ping {
 
         my ( $self, $arg_r ) = @_;
 
         my $logger = get_logger(__PACKAGE__);
-        $logger->debug('BEGIN');
 
         my $answer_hr = $self->xmlrpc(
             {
@@ -304,21 +279,14 @@ use Readonly;
             }
         );
 
-        $logger->debug('END');
-
         return;
     }
 
-   # +=======================================================================+
-   # || CipUX::RPC version test                                             ||
-   # +=======================================================================+
-
     sub test_version {
 
         my ( $self, $arg_r ) = @_;
 
         my $logger = get_logger(__PACKAGE__);
-        $logger->debug('BEGIN');
 
         my $answer_hr = $self->xmlrpc(
             {
@@ -355,21 +323,14 @@ use Readonly;
             }
         );
 
-        $logger->debug('END');
-
         return;
     }
 
-   # +=======================================================================+
-   # || CipUX::RPC sum 1 test                                               ||
-   # +=======================================================================+
-   #
     sub test_sum1 {
 
         my ( $self, $arg_r ) = @_;
 
         my $logger = get_logger(__PACKAGE__);
-        $logger->debug('BEGIN');
 
         my $answer_hr = $self->xmlrpc(
             {
@@ -404,21 +365,14 @@ use Readonly;
             }
         );
 
-        $logger->debug('END');
-
         return;
     }
 
-   # +=======================================================================+
-   # || CipUX::Task sum 2 test                                              ||
-   # +=======================================================================+
-
     sub test_sum2 {
 
         my ( $self, $arg_r ) = @_;
 
         my $logger = get_logger(__PACKAGE__);
-        $logger->debug('BEGIN');
 
         my $answer_hr = $self->xmlrpc(
             {
@@ -449,21 +403,14 @@ use Readonly;
             }
         );
 
-        $logger->debug('END');
-
         return;
     }
 
-   # +=======================================================================+
-   # || login function                                                      ||
-   # +=======================================================================+
-
     sub test_login {
 
         my ( $self, $arg_r ) = @_;
 
         my $logger = get_logger(__PACKAGE__);
-        $logger->debug('BEGIN');
 
         #1
         $self->manual_test( { title => 'CipUX login prompt exec' } );
@@ -482,21 +429,14 @@ use Readonly;
 
         $self->login( { login => $login, password => $password } );
 
-        $logger->debug('END');
-
         return;
     }
 
-   # +=======================================================================+
-   # || CipUX::Task list admin test                                         ||
-   # +=======================================================================+
-
     sub test_list_admin {
 
         my ( $self, $arg_r ) = @_;
 
         my $logger = get_logger(__PACKAGE__);
-        $logger->debug('BEGIN');
         $logger->debug('list admin accounts exec');
 
         my $answer_hr = $self->xmlrpc(
@@ -529,21 +469,14 @@ use Readonly;
             }
         );
 
-        $logger->debug('END');
-
         return;
     }
 
-   # +=======================================================================+
-   # || ttl                                                                 ||
-   # +=======================================================================+
-
     sub test_ttl {
 
         my ( $self, $arg_r ) = @_;
 
         my $logger = get_logger(__PACKAGE__);
-        $logger->debug('BEGIN');
 
         my $answer_hr = $self->xmlrpc(
             {
@@ -567,12 +500,11 @@ use Readonly;
                 mode    => 'compare',
                 minor   => 0,
                 major   => $got_ttl,
-                print   => 'TTL is ' . $got_ttl . ' minutes.',
+                print   => 'TTL is ' . $got_ttl . ' seconds.',
                 success => 'We got a TTL',
                 failure => 'We got no TTL'
             }
         );
-
         $self->record_test_result(
             {
                 start   => 1,
@@ -585,21 +517,14 @@ use Readonly;
             }
         );
 
-        $logger->debug('END');
-
         return;
     }
 
-   # +=======================================================================+
-   # || session                                                             ||
-   # +=======================================================================+
-
     sub test_session {
 
         my ( $self, $arg_r ) = @_;
 
         my $logger = get_logger(__PACKAGE__);
-        $logger->debug('BEGIN');
 
         sleep $A_LITTLE_WHILE;
 
@@ -628,42 +553,29 @@ use Readonly;
             }
         );
 
-        $logger->debug('END');
-
         return;
     }
 
-   # +=======================================================================+
-   # || cipux_task_{create|list|destroy}_student_account                    ||
-   # +=======================================================================+
-
+    #  cipux_task_{create|list|destroy}_student_account
     sub test_entity1 {
 
         my ( $self, $arg_r ) = @_;
 
         my $logger = get_logger(__PACKAGE__);
-        $logger->debug('BEGIN');
 
         foreach my $entity ( sort keys %{$ENTITY_OBJ_HR} ) {
             $self->big_entity_test(
                 { entity => $entity, obj => $ENTITY_OBJ_HR->{$entity} } );
         }
 
-        $logger->debug('END');
-
         return;
     }
 
-   # +=======================================================================+
-   # || log out                                                             ||
-   # +=======================================================================+
-
     sub test_logout {
 
         my ( $self, $arg_r ) = @_;
 
         my $logger = get_logger(__PACKAGE__);
-        $logger->debug('BEGIN');
 
         my $answer_hr = $self->xmlrpc(
             {
@@ -677,20 +589,14 @@ use Readonly;
             }
         );
 
-        $logger->debug('END');
-
         return;
     }
 
-   # +=======================================================================+
-   # || list users                                                          ||
-   # +=======================================================================+
     sub test_list_users {
 
         my ( $self, $arg_r ) = @_;
 
         my $logger = get_logger(__PACKAGE__);
-        $logger->debug('BEGIN');
 
         my $answer_hr = $self->xmlrpc(
             {
@@ -724,58 +630,40 @@ use Readonly;
             }
         );
 
-        $logger->debug('END');
-
         return;
     }
 
-   # +=======================================================================+
-   # || log in again                                                        ||
-   # +=======================================================================+
-
     sub test_login_again {
 
         my ( $self, $arg_r ) = @_;
 
         my $logger = get_logger(__PACKAGE__);
-        $logger->debug('BEGIN');
 
         $self->login( { login => $login, password => $password } );
 
-        $logger->debug('END');
-
         return;
     }
 
-   # +=======================================================================+
-   # || cipux_task_{create|list|destroy}_course_share                       ||
-   # +=======================================================================+
+    # cipux_task_{create|list|destroy}_course_share
 
     sub test_entity2 {
 
         my ( $self, $arg_r ) = @_;
 
         my $logger = get_logger(__PACKAGE__);
-        $logger->debug('BEGIN');
 
         $self->big_entity_test(
             { entity => 'course_share', obj => 'testcourse' } );
 
-        $logger->debug('END');
-
         return;
     }
 
-   # +=======================================================================+
-   # || cipux_task_{create|list|destroy}_ldap_orga_node                     ||
-   # +=======================================================================+
-
+    # cipux_task_{create|list|destroy}_ldap_orga_node
     sub test_create_list_destroy1 {
 
         my ( $self, $arg_r ) = @_;
 
         my $logger = get_logger(__PACKAGE__);
-        $logger->debug('BEGIN');
 
         $self->create_list_destroy_test(
             {
@@ -788,21 +676,16 @@ use Readonly;
             }
         );
 
-        $logger->debug('END');
-
         return;
     }
 
-   # +=======================================================================+
-   # || cipux_task_{create|list|destroy}_cat_module                         ||
-   # +=======================================================================+
+    # cipux_task_{create|list|destroy}_cat_module
 
     sub test_create_list_destroy2 {
 
         my ( $self, $arg_r ) = @_;
 
         my $logger = get_logger(__PACKAGE__);
-        $logger->debug('BEGIN');
 
         $self->create_list_destroy_test(
             {
@@ -825,42 +708,28 @@ use Readonly;
             }
         );
 
-        $logger->debug('END');
-
         return;
     }
 
-   # +=======================================================================+
-   # || plain_entity_test                                                   ||
-   # +=======================================================================+
-
     sub test_plain_entity {
 
         my ( $self, $arg_r ) = @_;
 
         my $logger = get_logger(__PACKAGE__);
-        $logger->debug('BEGIN');
 
         foreach my $entity ( keys %{$PLAIN_ENTITY_HR} ) {
             $self->plain_entity_test(
                 { entity_hr => $PLAIN_ENTITY_HR->{$entity} } );
         }
 
-        $logger->debug('END');
-
         return;
     }
 
-   # +=======================================================================+
-   # || Summary                                                             ||
-   # +=======================================================================+
-
     sub test_summary {
 
         my ( $self, $arg_r ) = @_;
 
         my $logger = get_logger(__PACKAGE__);
-        $logger->debug('BEGIN');
 
         $self->out($L);
         my $width   = $W - $BORDER;
@@ -881,14 +750,9 @@ use Readonly;
             $success_summary, $failure_summary;
         $self->out( '    +' . q{-} x ( $width + 1 ) . "+\n" );
 
-        $logger->debug('END');
-
         return;
     }
 
-   # +=======================================================================+
-   # || change_own_password                                                 ||
-   # +=======================================================================+
    #
    # out("="x76,"\n");
    # out("($test) connect to $url over SSL ...\n");
@@ -899,10 +763,6 @@ use Readonly;
    # $server->call('cipux_task_change_own_password',@args);
    # exit 0;
 
-   # +=======================================================================+
-   # || change_own_password                                                 ||
-   # +=======================================================================+
-   #
    # out("="x76,"\n");
    # out("($test) connect to $url over SSL ...\n");
    # out("    try to get the userlist with cipux_task_change_own_password\n");
@@ -919,7 +779,6 @@ use Readonly;
         my ( $self, $arg_r ) = @_;
 
         my $logger = get_logger(__PACKAGE__);
-        $logger->debug('BEGIN');
 
         my $login
             = exists $arg_r->{login}
@@ -1040,8 +899,6 @@ use Readonly;
             $ticket = $answer_hr->{cmdres_r}->{ticket};
         }
 
-        $logger->debug('END');
-
         return $answer_hr;
 
     }
@@ -1051,7 +908,6 @@ use Readonly;
         my ( $self, $arg_r ) = @_;
 
         my $logger = get_logger(__PACKAGE__);
-        $logger->debug('BEGIN');
 
         my $title
             = exists $arg_r->{title}
@@ -1067,8 +923,6 @@ use Readonly;
         $self->out($L);
         $self->out("($test) ==> $title{$test} <== (M) \n");
 
-        $logger->debug('END');
-
         return;
 
     }
@@ -1079,7 +933,6 @@ use Readonly;
         my ( $self, $arg_r ) = @_;
 
         my $logger = get_logger(__PACKAGE__);
-        $logger->debug('BEGIN');
 
         my $mode
             = exists $arg_r->{mode}
@@ -1174,8 +1027,6 @@ use Readonly;
             $self->out( q{    (} . $failure . ")\n" );
         }
 
-        $logger->debug('END');
-
         return $result;
     }
 
@@ -1184,7 +1035,6 @@ use Readonly;
         my ( $self, $arg_r ) = @_;
 
         my $logger = get_logger(__PACKAGE__);
-        $logger->debug('BEGIN');
 
         my $login
             = exists $arg_r->{login}
@@ -1260,29 +1110,24 @@ use Readonly;
 
         exit 1 if not $result;
 
-        $logger->debug('END');
-
         return;
 
     }
 
-   # +=======================================================================+
-   # || cipux_task_{create|list|destroy}_{$entity}                          ||
-   # +=======================================================================+
-   # cipux_task_create_{entity}
-   # cipux_task_change_{entity}_password
-   # cipux_task_add_member_to_{entity}
-   # cipux_task_list_members_of_{entity}
-   # cipux_task_remove_member_from_{entity}
-   # cipux_task_list_{entity}s
-   # cipux_task_destroy_{entity}
-   #
+    #  cipux_task_{create|list|destroy}_{$entity}
+    # cipux_task_create_{entity}
+    # cipux_task_change_{entity}_password
+    # cipux_task_add_member_to_{entity}
+    # cipux_task_list_members_of_{entity}
+    # cipux_task_remove_member_from_{entity}
+    # cipux_task_list_{entity}s
+    # cipux_task_destroy_{entity}
+    #
     sub big_entity_test {
 
         my ( $self, $arg_r ) = @_;
 
         my $logger = get_logger(__PACKAGE__);
-        $logger->debug('BEGIN');
 
         my $entity
             = exists $arg_r->{entity}
@@ -1305,10 +1150,7 @@ use Readonly;
             }
         );
 
-   # +-----------------------------------------------------------------------+
-   # || cipux_task_create_{ $entity }                                       ||
-   # +-----------------------------------------------------------------------+
-
+        # cipux_task_create_{ $entity }
         $logger->debug( 'RPC CREATE: cipux_task_create_' . $entity );
         my $answer_hr = $self->xmlrpc(
             {
@@ -1362,10 +1204,7 @@ use Readonly;
             }
         );
 
-   # +-----------------------------------------------------------------------+
-   # || cipux_task_change_{. $entity . }_password                           ||
-   # +-----------------------------------------------------------------------+
-
+        # cipux_task_change_{. $entity . }_password
         $logger->debug( 'RPC CHANGE PWD: change_' . $entity . '_password' );
         my $rnd_password = $self->random_password();
 
@@ -1373,22 +1212,22 @@ use Readonly;
             {
                 title => 'CipUX::Task 07 change password of ' 
                     . $entity . ' exec',
-                print    => 'operating on: ' . $obj,
-                url      => $url,
-                login    => $login,
-                ticket   => $ticket,
-                cmd      => 'cipux_task_change_' . $entity . '_password',
-                param_hr => { object => $obj, userPassword => $rnd_password },
+                print  => 'operating on: ' . $obj,
+                url    => $url,
+                login  => $login,
+                ticket => $ticket,
+                cmd    => 'cipux_task_change_' . $entity . '_password',
+
+               #param_hr => { object => $obj, userPassword => $rnd_password },
+                param_hr => { object => $obj, value => $rnd_password },
                 expect   => 'TRUE',
             }
         );
 
-   # +-----------------------------------------------------------------------+
-   # || cipux_task_add_member_{ $entity }                                   ||
-   # +-----------------------------------------------------------------------+
-   # cipux_task_add_member_to_course_share
-   # cipux_task_list_members_of_course_share
-   # cipux_task_remove_member_from_course_share
+        # cipux_task_add_member_{ $entity }
+        # cipux_task_add_member_to_course_share
+        # cipux_task_list_members_of_course_share
+        # cipux_task_remove_member_from_course_share
 
         my $member = 'testmember';
 
@@ -1431,7 +1270,8 @@ use Readonly;
         $self->record_test_result(
             {
                 start => 1,
-                title => 'CipUX::Task 10 add member to ' 
+                title => 'CipUX::Task 10 add ' 
+                    . $member . ' to ' 
                     . $entity
                     . ' result',
                 mode    => 'true',
@@ -1487,10 +1327,7 @@ use Readonly;
             }
         );
 
-   # +-----------------------------------------------------------------------+
-   # || cipux_task_destroy_{ $entity }                                      ||
-   # +-----------------------------------------------------------------------+
-
+        #  cipux_task_destroy_{ $entity }
         if ($destroy) {
             my $answer_hr = $self->xmlrpc(
                 {
@@ -1532,29 +1369,24 @@ use Readonly;
             }
         );
 
-        $logger->debug('END');
-
         return;
 
     }
 
-   # +=======================================================================+
-   # || plain_entity_test                                                   ||
-   # +=======================================================================+
-   # cipux_task_create_{entity}
-   # cipux_task_list_{entity}s
-   # cipux_task_addmodify_all_clients_of_netgroup  (add)
-   # cipux_task_list_{target}s_of_{entity}
-   # cipux_task_erase_clients_from_netgroup
-   # cipux_task_list_{target}s_of_{entity}
-   # cipux_task_destroy_{entity}
-   #
+    # plain_entity_test
+    # cipux_task_create_{entity}
+    # cipux_task_list_{entity}s
+    # cipux_task_addmodify_all_clients_of_netgroup  (add)
+    # cipux_task_list_{target}s_of_{entity}
+    # cipux_task_erase_clients_from_netgroup
+    # cipux_task_list_{target}s_of_{entity}
+    # cipux_task_destroy_{entity}
+    #
     sub plain_entity_test {
 
         my ( $self, $arg_r ) = @_;
 
         my $logger = get_logger(__PACKAGE__);
-        $logger->debug('BEGIN');
 
         my $e_hr
             = exists $arg_r->{entity_hr}
@@ -1593,9 +1425,7 @@ use Readonly;
             }
         );
 
-   # +-----------------------------------------------------------------------+
-   # || cipux_task_create_{ $entity }                                       ||
-   # +-----------------------------------------------------------------------+
+        # cipux_task_create_{ $entity }
 
         my $answer_hr = $self->xmlrpc(
             {
@@ -1639,12 +1469,10 @@ use Readonly;
             }
         );
 
-      # +--------------------------------------------------------------------+
-      # || cipux_task_add_member_{ $entity }                                ||
-      # +--------------------------------------------------------------------+
-      # cipux_task_addmodify_all_clients_of_netgroup
-      # cipux_task_list_clients_of_netgroup
-      # cipux_task_erase_clients_from_netgroup
+        #  cipux_task_add_member_{ $entity }
+        # cipux_task_addmodify_all_clients_of_netgroup
+        # cipux_task_list_clients_of_netgroup
+        # cipux_task_erase_clients_from_netgroup
 
        #cmd => 'cipux_task_add_'.$member_obj.'_to_' . $entity . $EMPTY_STRING,
         my $cmd
@@ -1726,9 +1554,7 @@ use Readonly;
             }
         );
 
-   # +-----------------------------------------------------------------------+
-   # || cipux_task_destroy_{ $entity }                                      ||
-   # +-----------------------------------------------------------------------+
+        # cipux_task_destroy_{ $entity }
 
         if ($destroy) {
             my $answer_hr = $self->xmlrpc(
@@ -1771,8 +1597,6 @@ use Readonly;
             }
         );
 
-        $logger->debug('END');
-
         return;
 
     }
@@ -1782,7 +1606,6 @@ use Readonly;
         my ( $self, $arg_r ) = @_;
 
         my $logger = get_logger(__PACKAGE__);
-        $logger->debug('BEGIN');
 
         my $testobj
             = exists $arg_r->{testobj}
@@ -1809,10 +1632,7 @@ use Readonly;
         $logger->debug("destroy: $destroy");
         $logger->debug("title  : $title");
 
-   # +-----------------------------------------------------------------------+
-   # || cipux_task_destroy_'.$entity.' (maybe)                              ||
-   # +-----------------------------------------------------------------------+
-
+        # cipux_task_destroy_'.$entity.' (maybe)
         # (1) first list, to see if it is there
         my $answer_hr = $self->xmlrpc(
             {
@@ -1869,8 +1689,6 @@ use Readonly;
             );
         }
 
-        $logger->debug('END');
-
         return;
 
     }
@@ -1880,7 +1698,6 @@ use Readonly;
         my ( $self, $arg_r ) = @_;
 
         my $logger = get_logger(__PACKAGE__);
-        $logger->debug('BEGIN');
 
         my $title
             = exists $arg_r->{title}
@@ -2005,8 +1822,6 @@ use Readonly;
             }
         );
 
-        $logger->debug('END');
-
         return;
     }
 
@@ -2026,7 +1841,6 @@ use Readonly;
         my ( $self, $arg_r ) = @_;
 
         my $logger = get_logger(__PACKAGE__);
-        $logger->debug('BEGIN');
 
         my $answer_hr
             = ( exists $arg_r->{answer_hr} )
@@ -2041,12 +1855,52 @@ use Readonly;
         my $task = $answer_hr->{cmd};
         $logger->debug("task: $task");
 
+        # This is only used for groups
+        # TODO: remove this and use CipUX::RPC::Client
+        my %target_attr = (
+            cipux_task_list_admin_accounts               => 'memberUid',
+            cipux_task_list_student_accounts             => 'memberUid',
+            cipux_task_list_teacher_accounts             => 'memberUid',
+            cipux_task_list_pupil_accounts               => 'memberUid',
+            cipux_task_list_lecturer_accounts            => 'memberUid',
+            cipux_task_list_assistant_accounts           => 'memberUid',
+            cipux_task_list_professor_accounts           => 'memberUid',
+            cipux_task_list_tutor_accounts               => 'memberUid',
+            cipux_task_list_members_of_role_account      => 'memberUid',
+            cipux_task_list_members_of_admin_account     => 'memberUid',
+            cipux_task_list_members_of_student_account   => 'memberUid',
+            cipux_task_list_members_of_teacher_account   => 'memberUid',
+            cipux_task_list_members_of_pupil_account     => 'memberUid',
+            cipux_task_list_members_of_lecturer_account  => 'memberUid',
+            cipux_task_list_members_of_assistant_account => 'memberUid',
+            cipux_task_list_members_of_professor_account => 'memberUid',
+            cipux_task_list_members_of_tutor_account     => 'memberUid',
+            cipux_task_list_course_shares                => 'uid',
+            cipux_task_list_class_shares                 => 'uid',
+            cipux_task_list_lecture_shares               => 'uid',
+            cipux_task_list_seminar_shares               => 'uid',
+            cipux_task_list_workshop_shares              => 'uid',
+            cipux_task_list_reading_shares               => 'uid',
+            cipux_task_list_studygroup_shares            => 'uid',
+            cipux_task_list_team_shares                  => 'uid',
+            cipux_task_list_tutorial_shares              => 'uid',
+            cipux_task_list_members_of_course_share      => 'memberUid',
+            cipux_task_list_members_of_class_share       => 'memberUid',
+            cipux_task_list_members_of_lecture_share     => 'memberUid',
+            cipux_task_list_members_of_seminar_share     => 'memberUid',
+            cipux_task_list_members_of_workshop_share    => 'memberUid',
+            cipux_task_list_members_of_reading_share     => 'memberUid',
+            cipux_task_list_members_of_studygroup_share  => 'memberUid',
+            cipux_task_list_members_of_team_share        => 'memberUid',
+            cipux_task_list_members_of_tutorial_share    => 'memberUid',
+        );
         my $target_attr
-            = ( exists $arg_r->{taget_attr} )
+            = ( defined $target_attr{$task} ) ? $target_attr{$task}
+            : ( exists $arg_r->{taget_attr} )
             ? $self->l( $arg_r->{target_attr} )
-            : ( defined $task_hr->{$task}->{return}->[0] )
-            ? $task_hr->{$task}->{return}->[0]
-            : $self->perr('target_attr');
+            : 'cn';
+
+        #$target_attr = $target_attr{$task}; # memberUid
 
         $logger->debug( 'answer_hr: ',
             { filter => \&Dumper, value => $answer_hr } );
@@ -2075,7 +1929,6 @@ use Readonly;
                 }
             }
         }
-        $logger->debug('END');
 
         if ( $member ne $EMPTY_STRING ) {
             return $result;
@@ -2100,7 +1953,7 @@ CipUX::RPC::Test::Client - libray for test clients
 
 =head1 VERSION
 
-version 3.4.0.6
+version 3.4.0.8
 
 =head1 USAGE
 
